@@ -16,6 +16,16 @@ public class SpecialObjectType extends ObjectType {
         this.setSignature(implementationType.getSignature());
     }
 
+    @Override
+    public ClassType getSuperclass() {
+        return implementationType.getSuperclass();
+    }
+
+    @Override
+    public ClassType[] getInterfaces() {
+        return implementationType.getInterfaces();
+    }
+
     @Override
     public Field getField(String name, int mask) {
         return implementationType.getField(name, mask);
