@@ -405,7 +405,7 @@ public class ApplyExp extends Expression
         Declaration rest = pushArgs(func_lambda, exp.args, fixed, incValues, comp);
         if (toArray)
             PrimProcedure.compileRestArg(rest.getType(), exp,
-                                         0, exp.args.length-1, comp);
+                                         0, fixed, comp);
         method = null;
       }
     else
@@ -525,12 +525,15 @@ public class ApplyExp extends Expression
                 arg.compileWithPosition(comp, Target.pushObject);
             if (mode == '@' || mode == ':') {
                 String mname = mode == '@' ? "addSequence" : "addArgList";
-                code.emitInvoke(typeArgListImpl.getDeclaredMethod(mname, 1));
+                code.emitInvoke(new Method(typeArgListImpl
+                                           .getDeclaredMethod(mname, 1),
+                                           Compilation.typeCallContext));
             } else if (mode == 'K' && ! useSetKeys) {
-                code.emitInvoke(Compilation.typeCallContext
-                                .getDeclaredMethod("addKey", 2));
+                code.emitInvoke(new Method(typeArgListImpl
+                                .getDeclaredMethod("addKey", 2),
+                                           Compilation.typeCallContext));
             } else {
-                code.emitInvoke(Compilation.typeCallContext
+              code.emitInvoke(Compilation.typeCallContext
                                 .getDeclaredMethod("addArg", 1));
             }
             if (mode == 'K' && useSetKeys
@@ -622,6 +625,8 @@ public class ApplyExp extends Expression
                     ptype = ((ArrayType) vtype).getComponentType();
             }
             args[i] = visitor.visitAndUpdate(args[i], ptype);
+            if (param != null)
+                param.noteValueFromApply(this, i);
             if (param != null && ! param.getFlag(Declaration.IS_REST_PARAMETER))
                 param = param.nextDecl();
         }
@@ -647,18 +652,23 @@ public class ApplyExp extends Expression
     out.writeSpaceFill();
     printLineColumn(out);
     func.print(out);
-    int firstKeyword = firstKeywordArgIndex-1;
-    for (int i = 0; args != null && i < args.length; ++i)
-      {
-	out.writeSpaceLinear();
-        Expression arg = args[i];
-        if (i >= firstKeyword && ((i - firstKeyword) & 1) == 0
-            && i < firstKeyword + 2 * numKeywordArgs
-            && arg.valueIfConstant() instanceof Keyword) {
-            out.print(arg.valueIfConstant().toString());
-        } else
-            arg.print(out);
-      }
+    if (args != null && args.length > 0) {
+        out.writeShowHideButton(true);
+        out.startHiderSection(true);
+        int firstKeyword = firstKeywordArgIndex-1;
+        for (int i = 0; i < args.length; ++i)
+        {
+            out.writeSpaceLinear();
+            Expression arg = args[i];
+            if (i >= firstKeyword && ((i - firstKeyword) & 1) == 0
+                && i < firstKeyword + 2 * numKeywordArgs
+                && arg.valueIfConstant() instanceof Keyword) {
+                out.print(arg.valueIfConstant().toString());
+            } else
+                arg.print(out);
+        }
+        out.endHiderSection();
+    }
     out.endLogicalBlock(")");
   }
 
