@@ -2,7 +2,7 @@ package gnu.expr;
 
 import gnu.lists.Consumer;
 import gnu.mapping.*;
-import gnu.text.Printable;
+import gnu.kawa.format.Printable;
 import java.io.*;
 
 /**
@@ -13,7 +13,7 @@ import java.io.*;
  *
  * @author Per Bothner
  */
-public class Keyword extends Symbol implements Printable, Externalizable {
+public class Keyword extends Symbol implements Externalizable {
     public static final Namespace keywordNamespace = Namespace.create();
 
     static {
@@ -80,12 +80,6 @@ public class Keyword extends Symbol implements Printable, Externalizable {
         return getName() + ':';
     }
 
-    public void print(Consumer out) {
-        Symbols.print(getName(), out);
-        out.write(':');
-
-    }
-
     /**
      * Search vals[0:offset-1] for a keyword.
      * Each key at vals[i] is followed by a value at keys[i+1].
