@@ -230,7 +230,8 @@ public class ModuleExp extends LambdaExp
             comp.process(Compilation.RESOLVED);
             comp.getMinfo().loadByStages(Compilation.WALKED);
 
-            if (msg != null ? messages.checkErrors(msg, 20) : messages.seenErrors())
+            if (msg != null ? messages.checkErrors(msg, Compilation.maxErrors())
+                : messages.seenErrors())
                 return null;
 
             if (! comp.mustCompile) {
@@ -388,6 +389,8 @@ public class ModuleExp extends LambdaExp
 
     public void allocChildClasses(Compilation comp) {
         declareClosureEnv();
+        if (! comp.usingCPStyle())
+            return;
         allocFrame(comp);
     }
 
@@ -486,7 +489,9 @@ public class ModuleExp extends LambdaExp
         if (mname == null) {
             if (fileName == null)
                 mname = "$unnamed_input_file$";
-            else if (fileName.equals("-") || fileName.equals("/dev/stdin"))
+            else if (fileName.equals("-")
+                     || fileName.equals("/dev/tty")
+                     || fileName.equals("/dev/stdin"))
                 mname = "$stdin$";
             else {
                 path = Path.valueOf(fileName);
