@@ -152,7 +152,7 @@ public abstract class LispLanguage extends Language
     Translator tr = (Translator) comp;
     ModuleExp mexp = tr.getModule();
     tr.resolveModule(mexp);
-    if (tr.subModuleMap != null) {
+    if (tr.subModuleMap != null && tr.mainClass != null) {
         String mainName = tr.mainClass.getName();
         ModuleInfo subinfo = tr.subModuleMap.get(mainName);
         if (subinfo != null
@@ -270,6 +270,7 @@ public abstract class LispLanguage extends Language
             types.put("constant-string", ClassType.make("java.lang.CharSequence"));
             types.put("abstract-string", ClassType.make("gnu.lists.CharSeq"));
             types.put("vector", LangObjType.vectorType);
+            types.put("gvector", LangObjType.gvectorType);
             types.put("string", LangObjType.stringType);
             types.put("empty-list", ClassType.make("gnu.lists.EmptyList"));
             types.put("sequence", LangObjType.sequenceType);
@@ -323,11 +324,26 @@ public abstract class LispLanguage extends Language
         return null;
     }
 
+    public static Type decodeArrayType(String name) {
+        int nlen = name.length();
+        if (nlen == 5)
+            return GenArrayType.generalInstance;
+        try {
+            int rank = Integer.parseInt(name.substring(5));
+            if (rank >= 0)
+                    return new GenArrayType(rank, Type.objectType);
+        } catch (Throwable ex) {
+        }
+        return null;
+    }
+
     @Override
     // FIXME: getNamedType is over-specialised....
     public Type getNamedType (String name) {
         // Initialise the type map if necessary.
         Type type = getTypeMap().get(name);
+        if (type == null && name.startsWith("array"))
+            return decodeArrayType(name);
         return (type != null) ? type : getPackageStyleType(name);
     }
 
