@@ -4,6 +4,7 @@ import gnu.mapping.*;
 import gnu.lists.*;
 import java.util.Iterator;
 import java.util.Stack;
+import gnu.expr.Keyword;
 
 /** Implement a Common Lisp "package" value.
  * 
@@ -37,8 +38,10 @@ public class LispPackage extends Namespace
   
   public static final LispPackage CLNamespace = 
       (LispPackage) valueOf("COMMON-LISP");
+  /* Symbols in the KeywordNamespace have type gnu.expr.Keyword. */
   public static final LispPackage KeywordNamespace =
       (LispPackage) valueOf("KEYWORD");
+  /* Symbols in the KawaNamespace have type SimpleSymbol. */
   public static final LispPackage KawaNamespace =
       (LispPackage) valueOf("KAWA");
   /* The class namespace is used to resolve Java class names in the context of
@@ -57,6 +60,7 @@ public class LispPackage extends Namespace
     /* Package CL inherits from this package to facilitate cross language
      * lookups */
     KawaNamespace.setExportedNamespace(EmptyNamespace);
+    KeywordNamespace.setExportedNamespace(Keyword.keywordNamespace);
     LispPackage.use(CLNamespace, KawaNamespace); // FIXME: Should be used from CL-USER
     LispPackage.use(CLNamespace, ClassNamespace);
     currentPackage.setGlobal(CLNamespace);
@@ -436,13 +440,13 @@ public class LispPackage extends Namespace
     for (NamespaceUse used = imported;  used != null;
 	 used = used.nextImported)
       {
-	sym = lookup(name, hash, false);
+	sym = used.imported.exported.lookup(name, hash, false);
 	if (sym != null)
 	  return sym;
       }
 
     if (create)
-      return add(Symbol.makeUninterned(name, this), hash); // Optimization
+      return createSymbol (name, hash); // Optimization
     else
       return null;
   }
@@ -527,7 +531,43 @@ public class LispPackage extends Namespace
       unintern(old);
     addToShadowingSymbols(symbol);
   }
-  
+
+    private Symbol createSymbol(String name, int hash) {
+	if (this == KeywordNamespace) return Keyword.make(name);
+	if (this == KawaNamespace) return Symbol.valueOf(name);
+	return add(Symbol.makeUninterned(name, this), hash);
+    }
+
+    public static boolean keywordp (Object x) {
+	return x instanceof Keyword;
+    }
+
+    public static Object symbolPackage (Object x) {
+	Object nil = CommonLisp.FALSE;
+	if (x == nil) return CLNamespace;
+	else if (x instanceof Symbol) {
+	    Namespace ns = ((Symbol)x).getNamespace();
+	    if (ns instanceof LispPackage) return (LispPackage)ns;
+	    else if (keywordp (x)) return KeywordNamespace;
+	    else if (x instanceof SimpleSymbol) return KawaNamespace;
+	    else return nil;
+	}
+	else throw new RuntimeException("argument not a symbol: " + x);
+    }
+
+    public static Object intern(String name, LispPackage pkg) {
+	name = name.intern();
+	if (name == "nil" && pkg == CLNamespace) return CommonLisp.FALSE;
+	return pkg.lookup(name, name.hashCode(), true);
+    }
+
+    /** Returns null if no package exists. */
+    public static LispPackage findPackage(String name) {
+	Namespace ns = Namespace.valueOfNoCreate(name);
+	if (ns instanceof LispPackage) return (LispPackage)ns;
+	return null;
+    }
+
   /**
    * Temporary stub until Kawa supports conditional restarts.
    */
