@@ -5,6 +5,7 @@ package gnu.kawa.xml;
 import gnu.lists.*;
 import gnu.xml.*;
 import gnu.mapping.*;
+import gnu.kawa.io.BinaryOutPort;
 import gnu.kawa.io.OutPort;
 import java.io.*;
 import java.util.Vector;
@@ -13,7 +14,7 @@ import java.util.Vector;
  * Used for both CGI scripts (default) and HttpServletResponse (future).
  */
 
-public class HttpPrinter extends FilterConsumer
+public class HttpPrinter extends PrintConsumer
 {
   Vector headers = new Vector();
   /** Used as output buffer if base is null. */
@@ -29,7 +30,7 @@ public class HttpPrinter extends FilterConsumer
   private int elementNesting;
 
   protected OutputStream ostream;
-  OutPort writer;
+  PrintConsumer writer;
 
   public HttpPrinter(OutputStream out)
   {
@@ -37,30 +38,17 @@ public class HttpPrinter extends FilterConsumer
     ostream = out;
   }
 
-  public HttpPrinter(OutPort out)
+  public HttpPrinter(PrintConsumer out)
   {
     super(null);
     writer = out;
   }
 
-  public static HttpPrinter make (OutPort out)
+  public static HttpPrinter make (PrintConsumer out)
   {
     return new HttpPrinter(out);
   }
 
-  private void writeRaw(String str)
-    throws java.io.IOException
-  {
-    if (writer != null)
-      writer.write(str);
-    else
-      {
-	int len = str.length();
-	for (int i = 0;  i < len;  i++)
-	  ostream.write((byte) str.charAt(i));
-      }
-  }
-
   protected void beforeNode ()
   {
     if (sawContentType == null)
@@ -132,7 +120,9 @@ public class HttpPrinter extends FilterConsumer
 	  style = "xhtml";
 	else if ("text/plain".equalsIgnoreCase(sawContentType))
 	  style = "plain";
-	base = XMLPrinter.make(writer, style);
+	XMLPrinter xout = XMLPrinter.make(writer, style);
+        out = xout;
+        base = xout;
         if (seenStartDocument == 0)
           {
             base.startDocument();
@@ -184,7 +174,22 @@ public class HttpPrinter extends FilterConsumer
 
   public void writeObject(Object v)
   {
-    if (v instanceof Consumable && ! (v instanceof UnescapedData))
+      if (v instanceof Blob) {
+          OutputStream outs =
+              writer instanceof BinaryOutPort
+              ? ((BinaryOutPort) writer).getOutputStream()
+              : ostream;
+          if (outs == null)
+              writer.write(v.toString());
+          else
+              try {
+                  ((Blob) v).writeTo(outs);
+              } catch (IOException ex) {
+                  throw new RuntimeException(ex);
+              }
+      }
+      else
+    if (v instanceof Consumable)
       ((Consumable) v).consume(this);
     else
       {
@@ -248,6 +253,12 @@ public class HttpPrinter extends FilterConsumer
       }
   }
 
+    public boolean ignoring() {
+        return base != null ? base.ignoring()
+            : writer != null ? writer.ignoring()
+            : false;
+    }
+
   /** Try to reset (delete) any response generated so far.
    * @param headersAlso if response headers should also be reset.
    * @return true on success, false if it's too late.
