@@ -1,4 +1,4 @@
-// Copyright (c) 2001, 2004  Per M.A. Bothner and Brainfood Inc.
+// Copyright (c) 2001, 2004, 2017  Per M.A. Bothner and Brainfood Inc.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.lists;
@@ -9,82 +9,62 @@ import java.io.Writer;
 
 /** Simple adjustable-length vector whose elements are 32-bit code points
  * Used for the Scheme string type.
+ * This isn't a regular SimpleVector because character indexing
+ * isn't a simple lookup.
+ * "Sub-range mode" is not used (at least not for Kawa Scheme):
+ * if you need an immutable sub-string, use an IString (or java.lang.String).
  * @author Per Bothner
  */
 
 public class FString extends AbstractCharVector<Char>
     implements Appendable, CharSeq, Consumable
 {
-  public FString ()
-  {
-    data = empty;
-  }
+    public FString() {
+        data = empty;
+    }
 
-  public FString (int num)
-  {
-    data = new char[num];
-  }
+    public FString(int num) {
+        data = new char[num];
+    }
 
     public FString(int num, int value) {
         data = new char[value < 0x10000 ? num : 2 * num];
-        insertRepeatedRaw(0, value, num);
+        if (value != 0 && num != 0)
+            insertRepeatedRaw(0, value, num);
     }
 
-  /** Create an FString from a char[].
-   * Note that this contructor does *not* copy the argument. */
-  public FString (char[] values)
-  {
-    data = values;
-  }
-
-  public FString (String str)
-  {
-    data = str.toCharArray();
-  }
-
-    public FString (StringBuilder buffer) {
-        this(buffer, 0, buffer.length());
+    /** Create an FString from a char[].
+     * Note that this contructor does *not* copy the argument. */
+    public FString(char[] values) {
+        data = values;
     }
 
-    public FString (StringBuilder buffer, int offset, int length) {
+    /** This constructor makes a copy. */
+    public FString(char[] buffer, int offset, int length) {
         data = new char[length];
-        if (length > 0)
-            buffer.getChars(offset, offset+length, data, 0);
+        System.arraycopy(buffer, offset, data, 0, length);
     }
 
-    public FString (StringBuffer buffer) {
-        this(buffer, 0, buffer.length());
-    }
-
-    public FString (StringBuffer buffer, int offset, int length) {
-        data = new char[length];
-        if (length > 0)
-            buffer.getChars (offset, offset+length, data, 0);
-    }
-
-  /** This constructor makes a copy. */
-  public FString (char[] buffer, int offset, int length)
-  {
-    data = new char[length];
-    System.arraycopy(buffer, offset, data, 0, length);
-  }
-
-  public FString(Sequence seq)
-  {
-    this.data = new char[seq.size()];
-    addAll(seq);
-  }
-
     public FString(CharSequence seq) {
         this(seq, 0, seq.length());
     }
 
+    /** Copy a substring of a CharSequence.
+     * @param offset - start offset in 16-bit char units
+     * @param length - length in 16-bit char units
+     */
     public FString(CharSequence seq, int offset, int length) {
         char[] data = new char[length];
         if (seq instanceof CharSeq)
             ((CharSeq) seq).getChars(offset, offset+length, data, 0);
-        else if (seq instanceof String)
-            ((String) seq).getChars(offset, offset+length, data, 0);
+        else if (seq instanceof String || seq instanceof IString) {
+            seq.toString().getChars(offset, offset+length, data, 0);
+            this.data = data;
+        }
+        else if (seq instanceof StringBuilder)
+            ((StringBuilder) seq).getChars(offset, offset+length, data, 0);
+        else if (seq instanceof StringBuffer)
+            ((StringBuffer) seq).getChars(offset, offset+length, data, 0);
         else {
             for (int i = length; --i >= 0; )
                 data[i] = seq.charAt(offset+i);
@@ -92,6 +72,85 @@ public class FString extends AbstractCharVector<Char>
         this.data = data;
     }
 
+    @Override
+    public int size() { return Strings.sizeInCodePoints(this); }
+
+    /** @param index measured in Unicode code points (characters)
+     */
+    @Override
+    public int effectiveIndex(int index) {
+        return relativeIndex(0, index);
+    }
+
+    private int relativeIndex(int start, int offset) {
+        if (isSubRange()) throw new InternalError();
+        int eff = start;
+        char prev = 0;
+        int dlen = data.length;
+        boolean beforeGap = ! isVerySimple();
+        int limit = beforeGap ? getGapStart() : dlen;
+        for (;;) {
+            if (eff >= limit && beforeGap) {
+                eff = getGapEnd();
+                limit = dlen;
+                beforeGap = false;
+            }
+            if (offset == 0)
+                return eff;
+            char ch = data[eff];
+            eff++;
+            if (! (prev >= 0xD800 && prev <= 0xDBFF
+                   && ch >= 0xDC00 && ch <= 0xDFFF))
+                offset--;
+            prev = ch;
+        }
+    }
+
+    @Override
+    public int createPos (int index, boolean isAfter) {
+        int eff = effectiveIndex(index);
+        // createPos ignores buffer gap; effectiveIndex includes it.
+        // (At least that is what we do for other SimpleVector types.)
+        if (isGapBuffer() && eff >= getGapEnd())
+            eff -= getGapSize();
+        return (eff << 1) | (isAfter ? 1 : 0);
+    }
+
+    @Override
+    protected int nextIndex(int ipos) {
+        int dlen = data.length;
+        int pos = ipos == -1 ? dlen : ipos >>> 1;
+        if (isGapBuffer() && pos >= getGapStart())
+            pos += getGapSize();
+        int eff = 0;
+        char prev = 0;
+        int index = 0;
+        boolean beforeGap = ! isVerySimple();
+        int limit = beforeGap ? getGapStart() : dlen;
+        for (;;) {
+            if (eff >= limit && beforeGap) {
+                eff = getGapEnd();
+                limit = dlen;
+                beforeGap = false;
+            }
+            if (eff == pos) {
+                return index;
+            }
+            char ch = data[eff];
+            eff++;
+            if (! (prev >= 0xD800 && prev <= 0xDBFF
+                   && ch >= 0xDC00 && ch <= 0xDFFF))
+                index++;
+            prev = ch;
+        }
+    }
+    public int createRelativePos(int pos, int delta, boolean isAfter) {
+        if (delta >= 0 && pos >= 0) {
+            return (relativeIndex(pos >>> 1, delta) << 1) | (isAfter ? 1 : 0);
+        }
+        return super.createRelativePos(pos, delta, isAfter);
+    }
+
     /** Create a empty string, but with a given initial buffer size. */
     public static FString alloc(int sz) {
         if (sz > MAX_GAP_SIZE)
@@ -102,13 +161,21 @@ public class FString extends AbstractCharVector<Char>
     }
 
     public final Char getRaw(int index) {
-        throw unsupported("getRaw "+index);
+        int ch1 = data[index];
+        if (ch1 >= 0xD800 && ch1 <= 0xDBFF) {
+            char ch2 = data[index+1];
+            if (ch2 >= 0xDC00 && ch2 <= 0xDFFF)
+                ch1 = ((ch1 - 0xD800) << 10) + (ch2 - 0xDC00) + 0x10000;
+        }
+        return Char.valueOf(ch1);
     }
 
+    /** @param index offset in character units (Unicode code points) */
     public final Char get(int index) {
-        return Char.valueOf(characterAt(index));
+        return Char.valueOf(Strings.indexByCodePoints(this, index));
     }
 
+    /** @param fromChar offset in 16-bit code units */
     public int indexOf(int ch, int fromChar) {
         char c1, c2;
         if (ch >= 0x10000) {
@@ -118,7 +185,7 @@ public class FString extends AbstractCharVector<Char>
             c1 = 0;
             c2 = (char) ch;
         }
-        int sz = size();
+        int sz = length();
         char prev = 0;
         for (int i = fromChar;  i < sz; i++) {
             char cur = charAt(i);
@@ -133,6 +200,7 @@ public class FString extends AbstractCharVector<Char>
         return -1;
     }
 
+    /** @param fromChar offset in 16-bit code units */
     public int lastIndexOf(int ch, int fromChar) {
         char c1, c2;
         if (ch >= 0x10000) {
@@ -153,21 +221,26 @@ public class FString extends AbstractCharVector<Char>
         return -1;
     }
 
+    /** @param index offset in character units (Unicode code points) */
     public Char set(int index, Char value) {
         checkCanWrite();
+        index = Character.offsetByCodePoints(this, 0, index);
         Char old = Char.valueOf(characterAt(index));
         setCharacterAt(index, value.intValue());
         return old;
     }
 
+    /** @param index offset in character units (Unicode code points) */
     @Override
     public final void setRaw(int index, Char value) {
+        index = Character.offsetByCodePoints(this, 0, index);
         setCharacterAt(index, value.intValue());
     }
 
+    /** @param index offset in 16-bit code units */
     public final int characterAt(int index) {
         // The following uses charAt, which handles adjusting for indexes.
-        return Strings.characterAt(this, 0, size(), index);
+        return Strings.characterAt(this, 0, length(), index);
     }
 
   /** Return a char[] contain the characters of this string.
@@ -177,7 +250,7 @@ public class FString extends AbstractCharVector<Char>
   {
     if (isVerySimple())
         return data;
-    int seq_length = size();
+    int seq_length = length();
     char[] arr = new char[seq_length];
     for (int i = 0;  i < seq_length;  i++)
         arr[i] = charAt(i);
@@ -201,7 +274,7 @@ public class FString extends AbstractCharVector<Char>
   public boolean addAll (CharSequence s)
   {
     int ssize = s.length();
-    int sz = size();
+    int sz = length();
     addSpace(sz, ssize); 
     if (s instanceof String)
       ((String) s).getChars(0, ssize, data, sz);
@@ -244,7 +317,7 @@ public class FString extends AbstractCharVector<Char>
    */
   public void addAllStrings(Object[] args, int startIndex)
   {
-    int sz = size();
+    int sz = length();
     int count = 0;
     for (int i = startIndex; i < args.length; ++i)
       {
@@ -261,7 +334,7 @@ public class FString extends AbstractCharVector<Char>
   }
   
     public String toString() {
-        return substring(0, size());
+        return substring(0, length());
     }
 
     public String substring(int start, int end) {
@@ -280,11 +353,11 @@ public class FString extends AbstractCharVector<Char>
 
     public void setCharAt(int index, char ch) {
         checkCanWrite(); // FIXME maybe inline and fold into following
-        data[effectiveIndex(index)] = ch;
+        data[super.effectiveIndex(index)] = ch;
     }
 
     public void setCharacterAt(int index, int ch) {
-        int sz = size();
+        int sz = length();
         if (index < 0 || index >= sz)
             throw new StringIndexOutOfBoundsException(index);
         char old1 = charAt(index);
@@ -311,6 +384,7 @@ public class FString extends AbstractCharVector<Char>
     /** Replace a substring of this string with another.
      * The two strings may have different lengths, so this
      * generalizes insertion and deletion.
+     * All indexes are code-unit (16-bit char) offsets.
      */
     public void replace(CharSequence src, int srcStart, int srcEnd,
                         int dstStart, int dstEnd) {
@@ -335,7 +409,9 @@ public class FString extends AbstractCharVector<Char>
                 return;
             }
         }
-        if (! Sequences.copyInPlaceIsSafe(this, src)) {
+        if (src != this
+            && ! (src instanceof IString || src instanceof String)
+            && ! Sequences.copyInPlaceIsSafe(this, src)) {
             src = src.subSequence(srcStart, srcEnd).toString();
             srcEnd = srcLength;
             srcStart = 0;
@@ -344,10 +420,17 @@ public class FString extends AbstractCharVector<Char>
             delete(dstEnd+grow, dstEnd);
         else if (grow > 0)
             setGapBounds(getGapStart()+grow, getGapEnd());
-        int i = dstStart;
-        int j = srcStart;
-        for (; j < srcEnd; i++, j++) {
-            data[i] = src.charAt(j);
+
+        if (srcStart < dstStart) {
+            int i = dstStart + srcLength;
+            int j = srcEnd;
+            while (--j >= srcStart)
+                data[--i] = src.charAt(j);
+         } else {
+            int i = dstStart;
+            int j = srcStart;
+            for (; j < srcEnd; i++, j++)
+                data[i] = src.charAt(j);
         }
     }
 
@@ -356,24 +439,6 @@ public class FString extends AbstractCharVector<Char>
     data[index] = ch;
   }
 
-    /** Set all the elements to a given character. */
-    public final void fill(char ch) {
-        fill(0, size(), ch);
-    }
-
-    public void fill(int fromIndex, int toIndex, char value) {
-        if (fromIndex < 0 || toIndex > size())
-            throw new IndexOutOfBoundsException();
-        if (isVerySimple()) { // Minor optimization
-            char[] d = data; // Move to local to help optimizer.
-            for (int i = fromIndex;  i < toIndex;  i++)
-                d[i] = value;
-        } else {
-            for (int i = fromIndex;  i < toIndex;  i++)
-                setCharAt(i, value);
-        }
-    }
-
     public void insertRepeated(int where, int value, int count) {
         addSpace(where, value < 0x10000 ? count : 2 * count);
         insertRepeatedRaw(where, value, count);
@@ -425,16 +490,11 @@ public class FString extends AbstractCharVector<Char>
 
     public String getTag() { return "c32"; }
 
-  public void consume(Consumer out)
-  {
-    out.write(data, 0, data.length);
-  }
-
     public void consumePosRange(int iposStart, int iposEnd, Consumer out) {
         if (out.ignoring())
             return;
-        int i = nextIndex(iposStart);
-        int end = nextIndex(iposEnd);
+        int i = iposStart >>> 3;
+        int end = iposEnd == -1 ? length() : iposEnd >>> 1;
         while (i < end) {
             long result = getSegment(i);
             int where = (int) result;
@@ -445,7 +505,7 @@ public class FString extends AbstractCharVector<Char>
     }
 
     public FString append(char c) {
-        int sz = size();
+        int sz = length();
         addSpace(sz, 1);
         char[] d = data;
         d[sz] = c;
@@ -461,7 +521,7 @@ public class FString extends AbstractCharVector<Char>
             return this;
         else
             delta = 2;
-        int sz = size();
+        int sz = length();
         addSpace(sz, delta);
         char[] d = data;
         if (delta > 1) {
@@ -482,7 +542,7 @@ public class FString extends AbstractCharVector<Char>
         if (csq == null)
             csq = "null";
         int len = end - start;
-        int sz = size();
+        int sz = length();
         addSpace(sz, len);
         char[] d = data;
         if (csq instanceof String)
@@ -502,8 +562,62 @@ public class FString extends AbstractCharVector<Char>
             appendCharacter(((gnu.text.Char) obj).intValue());
         else if (obj instanceof java.lang.Character)
             appendCharacter(((java.lang.Character) obj).charValue());
+        else {
+            CharSequence str = obj instanceof CharSequence ? (CharSequence) obj
+                : obj.toString();
+            append(str, 0, str.length());
+        }
+        return this;
+    }
+
+    public FString prependCharacter(int c) {
+        int delta;
+        if (c < 0x10000)
+            delta = 1;
+        else if (c == Char.IGNORABLE_CHAR)
+            return this;
         else
-            append(obj.toString());
+            delta = 2;
+        int sz = length();
+        gapReserve(0, delta);
+        int p = getGapEnd()-delta;
+        setGapBounds(getGapStart(), p);
+        char[] d = data;
+        if (delta > 1) {
+            d[p++] = (char) (((c - 0x10000) >> 10) + 0xD800);
+            c = (c & 0x3FF) + 0xDC00;
+        }
+        d[p++] = (char) c;
+        return this;
+    }
+
+    public FString prepend(CharSequence str, int start, int end) {
+        int len = end - start;
+        gapReserve(0, len);
+        int p = getGapEnd()-len;
+        setGapBounds(getGapStart(), p);
+        char[] d = data;
+        if (str instanceof String)
+            ((String) str).getChars(start, end, d, p);
+        else if (str instanceof CharSeq)
+            ((CharSeq) str).getChars(start, end, d, p);
+        else {
+            for (int i = start; i < end;  i++)
+                d[p++] = str.charAt(i);;
+        }
+        return this;
+    }
+
+    public FString prepend(Object obj) {
+        if (obj instanceof gnu.text.Char)
+            prependCharacter(((gnu.text.Char) obj).intValue());
+        else if (obj instanceof java.lang.Character)
+            prependCharacter(((java.lang.Character) obj).charValue());
+        else {
+            CharSequence str = obj instanceof CharSequence ? (CharSequence) obj
+                : obj.toString();
+            prepend(str, 0, str.length());
+        }
         return this;
     }
 
@@ -527,6 +641,6 @@ public class FString extends AbstractCharVector<Char>
     }
 
     public void writeTo(Appendable dest) throws IOException {
-        writeTo(0, size(), dest);
+        writeTo(0, length(), dest);
     }
 }
