@@ -1,5 +1,11 @@
 package gnu.mapping;
 
+/* #ifdef use:java.lang.invoke */
+import java.lang.invoke.*;
+/* #else */
+// import gnu.mapping.CallContext.MethodHandle; 
+/* #endif */
+
 /**
  * Abstract class for 0-argument procedures.
  * @author	Per Bothner
@@ -7,15 +13,13 @@ package gnu.mapping;
 
 public abstract class Procedure0 extends Procedure
 {
-  public Procedure0 ()
-  {
-    super();
-  }
+    public Procedure0() {
+        super(false, Procedure0.applyToObject);
+    }
 
-  public Procedure0 (java.lang.String n)
-  {
-    super(n);
-  }
+    public Procedure0(String name) {
+        super(false, Procedure0.applyToObject, name);
+    }
 
   public int numArgs() { return 0; }
 
@@ -48,4 +52,14 @@ public abstract class Procedure0 extends Procedure
       throw new WrongArguments(this, args.length);
     return apply0 ();
   }
+
+    public static Object applyToObject(Procedure proc, CallContext ctx)
+    throws Throwable {
+        if (ctx.checkDone() == 0)
+            return proc.apply0();
+        return ctx;
+    }
+
+    public static final MethodHandle applyToObject
+        = lookupApplyHandle(Procedure0.class, "applyToObject");
 }
