@@ -1,5 +1,11 @@
 package gnu.mapping;
 
+/* #ifdef use:java.lang.invoke */
+import java.lang.invoke.*;
+/* #else */
+// import gnu.mapping.CallContext.MethodHandle; 
+/* #endif */
+
 /**
  * Abstract class for 2-argument Scheme procedures.
  * Extensions must provide apply2.
@@ -8,15 +14,13 @@ package gnu.mapping;
 
 public abstract class Procedure2 extends Procedure
 {
+    public Procedure2() {
+        super(false, Procedure2.applyToObject);
+    }
 
-  public Procedure2(java.lang.String n)
-  {
-    super(n);
-  }
-  public Procedure2()
-  {
-    super();
-  }
+    public Procedure2(String name) {
+        super(false, Procedure2.applyToObject, name);
+    }
 
   public int numArgs() { return 0x2002; }
 
@@ -49,4 +53,16 @@ public abstract class Procedure2 extends Procedure
       throw new WrongArguments(this, args.length);
     return apply2 (args[0], args[1]);
   }
+
+    public static Object applyToObject(Procedure proc, CallContext ctx)
+    throws Throwable {
+        Object arg0 = ctx.getNextArg();
+        Object arg1 = ctx.getNextArg();
+        if (ctx.checkDone() == 0)
+            return proc.apply2(arg0, arg1);
+        return ctx;
+    }
+
+    public static final MethodHandle applyToObject
+        = lookupApplyHandle(Procedure2.class, "applyToObject");
 }
