@@ -126,9 +126,15 @@ public class DComplex extends Complex implements Externalizable
     return ((Numeric)y).divReversed(this);
   }
 
-  public static DComplex power (double x_re, double x_im,
-				double y_re, double y_im)
+  public static Complex power (double x_re, double x_im,
+                               double y_re, double y_im)
   {
+    if (x_re == 0.0 && x_im == 0.0) {
+      if (y_re > 0.0)
+        return DFloNum.zero();
+      else if (y_re == 0.0 && y_im == 0.0)
+        return DFloNum.valueOf(1.0);
+    }
     double h;
     /* #ifdef JAVA5 */
     h = Math.hypot(x_re, x_im);
@@ -156,7 +162,7 @@ public class DComplex extends Complex implements Externalizable
   // The code below is adapted from f2c's libF77, and is subject to this
   // copyright:
  
-  /****************************************************************
+  /*
     Copyright 1990, 1991, 1992, 1993 by AT&T Bell Laboratories and Bellcore.
  
     Permission to use, copy, modify, and distribute this software
@@ -177,7 +183,7 @@ public class DComplex extends Complex implements Externalizable
     in an action of contract, negligence or other tortious action,
     arising out of or in connection with the use or performance of
     this software.
-    ****************************************************************/
+   */
 
   public static DComplex div (double x_re, double x_im,
 			      double y_re, double y_im)
