@@ -37,8 +37,10 @@ public class SourceError extends SourceLocator.Simple
     }
 
     public SourceError(char severity, SourceLocator location, String message) {
-        this(severity, location.getFileName(), location.getLineNumber(),
-             location.getColumnNumber(), message);
+        this.severity = severity;
+        this.filename = location.getFileName();
+        this.position = SourceMapper.simpleEncode(location);
+        this.message = message;
     }
 
     /** Create a new SourceError using the current line/column from
@@ -68,10 +70,29 @@ public class SourceError extends SourceLocator.Simple
         appendTo(buffer, stripDirectories, null);
         return buffer.toString ();
     }
+
+    public static void appendEscaped(Appendable out, CharSequence str)
+        throws java.io.IOException {
+        int len = str.length();
+        for (int i = 0; i < len; i++) {
+            char ch = str.charAt(i);
+            if (ch == '<' || ch == '>' || ch == '&') {
+                out.append(ch == '<' ? "&lt;"
+                           : ch == '>' ? "&gt;"
+                           : "&amp;");
+            } else
+                out.append(ch);
+        }
+    }
    
     public void appendTo(Appendable out, boolean stripDirectories,
                          String newLine) {
+        boolean isDomTerm = gnu.kawa.io.CheckConsole.forDomTerm(out);
         try {
+            if (isDomTerm)
+                out.append("\033]44;"+(isRepl() ? "repl: true" : "")+"\007");
+            if (isDomTerm)
+                out.append("\033[44;0u");
             String fname;
             if (filename == null)
                 fname = "<unknown>";
@@ -80,17 +101,46 @@ public class SourceError extends SourceLocator.Simple
                 if (stripDirectories)
                     fname = new File(fname).getName();
             }
-            out.append(fname);
+            StringBuilder position = new StringBuilder();
+            String endpos = "";
             int line = getStartLine();
             int column = getStartColumn();
             if (line > 0 || column > 0) {
-                out.append(':');
-                out.append(Integer.toString(line));
+                position.append(Integer.toString(line));
                 if (column > 0) {
-                    out.append(':');
-                    out.append(Integer.toString(column));
+                    position.append(':');
+                    position.append(Integer.toString(column));
                 }
-                // FIXME show end position if non-empty
+                int eline = getEndLine();
+                int ecolumn = getEndColumn();
+                if (eline > 0 && ecolumn > 0 && column > 0) {
+                    if (line == eline && ecolumn > column) {
+                        endpos = "-" + ecolumn;
+                    } else if (eline > line) {
+                        endpos = "-" + eline + ":" + ecolumn;
+                    }
+                }
+            }
+            if (isDomTerm) {
+                out.append("\033]72;");
+                out.append("<a class='subtle' href='");
+                out.append(new File(fname).toURI().toString());
+                if (position.length() > 0) {
+                    out.append("#position=");
+                    out.append(position);
+                    out.append(endpos);
+                }
+                out.append("'>");
+                appendEscaped(out, fname);
+            }
+            else
+                out.append(fname);
+            if (position.length() > 0) {
+                out.append(':');
+                out.append(position);
+            }
+            if (isDomTerm) {
+                out.append("</a>\007");
             }
             out.append(": ");
             if (severity == 'w')
@@ -112,6 +162,8 @@ public class SourceError extends SourceLocator.Simple
                     out.append(stackTrace[i].toString());
                 }
             }
+            if (isDomTerm)
+                out.append("\033[44;0u");
             if (newLine != null)
                 out.append(newLine);
         } catch (java.io.IOException ex) {
