@@ -5,6 +5,7 @@ package kawa.lang;
 import gnu.mapping.*;
 import gnu.expr.*;
 import gnu.text.SourceMessages;
+import gnu.kawa.io.InPort;
 import gnu.lists.*;
 
 /* This implements the R5RS "eval" procedure. */
@@ -21,7 +22,7 @@ public class Eval
     else
       {
 	body = new PairWithPosition(sexpr, LList.Empty);
-	body.setFile("<eval>");
+	body.setFile(InPort.evalPathname);
       }
     evalBody(body, env, new SourceMessages(), ctx);
   }
@@ -78,7 +79,9 @@ public class Eval
         // to do the right thing.
         tr.setState(Compilation.BODY_PARSED-1);
         tr.setSharedModuleDefs(true);
-	ModuleExp mod = tr.pushNewModule((String) null);
+	ModuleExp mod = tr.pushNewModule(null);
+        if (env == Environment.user())
+            mod.setFlag(ModuleExp.INTERACTIVE);
         Compilation saveComp = Compilation.setSaveCurrent(tr);
         try
           {
