@@ -1,6 +1,9 @@
 (module-name <kawa.lib.rnrs.programs>)
 (module-export command-line exit emergency-exit)
-(require <kawa.lib.prim_syntax>)
+(require <kawa.lib.prim_imports>)
+(require <kawa.lib.lists>)
+(require kawa.lib.std_syntax)
+(require kawa.lib.syntax)
 
 (define (command-line) :: list
   (let* ((rest ; Actual arguments, not including "argv[0]" equivalent.
@@ -18,12 +21,13 @@
                       ;; Try looking at entire command line.
                       (let* ((raw1 (java.lang.System:getProperty
                                     "kawa.command.line"))
-                             (raw (if (eq? raw1 #!null)
+                             (raw ::java.lang.String
+                                  (if (eq? raw1 #!null)
                                       (let ((raw2
                                              (java.lang.System:getProperty
                                               "sun.java.command")))
                                         (if (eq? raw2 #!null) #!null
-                                            ("java ":concat raw2)))
+                                            &{java &[raw2]}))
                                       raw1)))
                         (if (eq? raw #!null) #!null
                             ;; Strip off the tail of the property value that
