@@ -1,8 +1,9 @@
 (module-name <kawa.lib.rnrs.sorting>)
 (module-export list-sort vector-sort vector-sort!)
-;;(import (srfi :95 sorting-and-merging))
 ;(import (kawa lib srfi95))
-(require <kawa.lib.srfi95>)
+(import (scheme base)
+        (kawa lib srfi 95 "kawa/lib/srfi/95.scm"))
+;(require <kawa.lib.srfi.95>)
 
 (define (list-sort less? list)
   (%sort-list (append list '()) less? #f))
