@@ -6,8 +6,14 @@
 	       string-upcase string-downcase string-titlecase string-foldcase
 	       string-normalize-nfd string-normalize-nfkd
 	       string-normalize-nfc string-normalize-nfkc)
+
+(require <kawa.lib.prim_imports>)
+(require <kawa.lib.std_syntax>)
 (require <kawa.lib.characters>)
+(require kawa.lib.exceptions)
 (require <kawa.lib.compile_misc>)
+(import (only kawa.lib.kawa.mstrings
+         string-upcase string-downcase string-titlecase string-foldcase))
 
 (define (char-upcase (ch :: character)) :: character
   (integer->char (java.lang.Character:toUpperCase (char->integer ch))))
@@ -47,18 +53,6 @@
 (define (char-general-category (ch :: character)) :: symbol
   (gnu.kawa.functions.UnicodeUtils:generalCategory (char->integer ch)))
 
-(define (string-upcase (str :: string)) :: string
-  (gnu.lists.FString ((str:toString):toUpperCase java.util.Locale:ENGLISH)))
-
-(define (string-downcase (str :: string)) :: string
-  (gnu.lists.FString ((str:toString):toLowerCase java.util.Locale:ENGLISH)))
-
-(define (string-titlecase (str :: string)) :: string
-  (gnu.lists.FString (gnu.kawa.functions.UnicodeUtils:capitalize str)))
-
-(define (string-foldcase (str :: string)) :: string
-  (gnu.lists.FString (gnu.kawa.functions.UnicodeUtils:foldCase str)))
-
 (define-syntax string-normalize
   (syntax-rules ()
     ((string-normalize str kind)
