@@ -69,14 +69,14 @@ public class repl extends Procedure0or1 {
 
     public static void printOption(PrintStream out,
                                    String option, String doc) {
-        out.print(" ");
-        out.print(option);
-
-        int len = option.length() + 1;
-        for (int i = 0; i < 30 - len; ++i)
-            out.print(" ");
-        out.print(" ");
-        out.println(doc);
+        StringBuilder buf = new StringBuilder(" ");
+        buf.append(option);
+        buf.append(' ');
+        for (int i = 30 - option.length(); --i >= 0; )
+            buf.append(' ');
+        buf.append(' ');
+        buf.append(doc);
+        out.println(buf);
     }
   
     public static void printOptions(PrintStream out) {
@@ -259,6 +259,16 @@ public class repl extends Procedure0or1 {
             checkDomTerm();
         for ( ;  iArg < maxArg;  )  {
             String arg = args[iArg++];
+            if (arg.equals("--langserver")) {
+                try {
+                    Class.forName("kawa.langserver.KawaLanguageServer")
+                        .getMethod("main", String[].class)
+                        .invoke(null, (Object) args);
+                } catch (Throwable ex) {
+                    WrappedException.rethrow(ex);
+                }
+                return -1;
+            }
             if (arg.equals ("-c") || arg.equals ("-e")) {
                 if (iArg == maxArg)
                     bad_option(arg);
@@ -629,7 +639,7 @@ public class repl extends Procedure0or1 {
                 System.out.print("Kawa ");
                 System.out.print(Version.getVersion());
                 System.out.println();
-                System.out.println("Copyright (C) 2017 Per Bothner");
+                System.out.println("Copyright (C) 2020 Per Bothner");
                 something_done = true;
             } else if (arg.startsWith("-D")) {
                 int eq = arg.indexOf('=');
@@ -894,7 +904,7 @@ public class repl extends Procedure0or1 {
             }
             if ("qtdomterm".equals(command))
                 command = "browser=qtdomterm --connect localhost:%W";
-            Object result = Class.forName("kawa.DomTermBackend")
+            Object result = Class.forName("kawa.DomTermServer")
                 .getMethod("startDomTermConsole", String.class)
                 .invoke(null, command);
             return result == null ? null : result.toString();
