@@ -1,6 +1,6 @@
 ;; -*- coding: utf-8 -*-
 
-(test-begin "libs" 260)
+(test-begin "libs" 278)
 
 (test-begin "vectors")
 (test-equal '(dah dah didah)
@@ -65,6 +65,12 @@
 (test-equal #u8(#xCE #xBB) (string->utf8 "λ"))
 (test-equal bytes1 (string->utf8 lambda-string))
 (test-equal lambda-string (utf8->string bytes1))
+(! hellox-str "Hæll◉ 😂!")
+(! hellox-utf8 #u8(#x48 #xc3 #xa6 #x6c #x6c #xe2 #x97 #x89
+                   #x20 #xf0 #x9f #x98 #x82 #x21))
+(test-equal hellox-utf8 (string->utf8 hellox-str))
+(test-equal hellox-str (utf8->string hellox-utf8))
+
 (test-end)
 
 (import (srfi :2 and-let*))
@@ -113,7 +119,7 @@
 (test-equal '("loc1" "uri1" "pre")
             (symbol-parts (symbol "loc1" "uri1" "pre")))
 (test-equal '("loc2" "uri2" "pre")
-            (symbol-parts (symbol 'loc2 'uri2 'pre)))
+            (symbol-parts (symbol "loc2" "uri2" "pre")))
 (test-equal '("loc3" "uri3" "pre")
             (symbol-parts (apply symbol (list "loc3" "uri3" "pre"))))
 (test-equal '("loc4" "uri4" "")
@@ -488,6 +494,30 @@
 
 (test-end "rnrs-lists")
 
+(test-begin "arglists")
+(import (kawa arglist))
+(let ()
+  (! a1 [10 11 12])
+  (test-equal 3 (arglist-arg-count a1))
+  (! a2 (arglist 2 k1: "K1" k2: "K2" @a1))
+  (test-equal 6 (arglist-arg-count a2))
+  (test-equal "k2" (arglist-key-ref a2 2))
+  (test-equal "K2" (arglist-arg-ref a2 2))
+  (test-equal 11 (arglist-arg-ref a2 4))
+  (test-equal 2 (arglist-key-index a2 "k2"))
+  (test-equal "K2" (arglist-key-value a2 "k2" "none"))
+  (test-equal -1 (arglist-key-index a2 "k3"))
+  (test-equal "none" (arglist-key-value a2 "k3" "none"))
+  (let ((out (open-output-string)))
+    (arglist-walk a2
+                  (lambda (key arg)
+                    (if key (format out "{~a: ~w}" key arg)
+                        (format out "{~w}" arg))))
+    (test-equal "{2}{k1: \"K1\"}{k2: \"K2\"}{10}{11}{12}"
+                (get-output-string out)))
+  )
+(test-end "arglists")
+
 (test-begin "strings")
 (import (kawa string-cursors))
 
@@ -616,36 +646,36 @@
                  (lambda (c) (java.lang.Character:title-case?
                               (char->integer c))) char-set:full)))
 
-;; Some of these tests only succeed on Java 8 (or later), which
-;; supports Unicode 6.2. On earlier Javas, the java.lang.Character
+;; Some of these tests only succeed on Java 9 (or later), which
+;; supports Unicode 8.0. On earlier Javas, the java.lang.Character
 ;; predicates will disagree with the char-set definitions.
-(define-syntax expect-fail-unless-unicode-6_2
+(define-syntax expect-fail-unless-unicode-8
   (syntax-rules ()
     ((_ count)
-     (cond-expand (java-8)
+     (cond-expand (java-9)
                   (else (test-expect-fail count))))))
 (define-syntax expect-fail-unless
   (syntax-rules ()
     ((_ version count)
      (cond-expand (version)
                   (else (test-expect-fail count))))))
-(expect-fail-unless java-8 6)
-(test-equal #t (char-set=               ; only on Java 8
+(expect-fail-unless java-9 6)
+(test-equal #t (char-set=               ; only on Java 9
                 char-set:lower-case
                 (char-set-filter
                  (lambda (c) (java.lang.Character:lower-case?
                               (char->integer c))) char-set:full)))
-(test-equal #t (char-set=               ; only on Java 8
+(test-equal #t (char-set=               ; only on Java 9
                 char-set:upper-case
                 (char-set-filter
                  (lambda (c) (java.lang.Character:upper-case?
                               (char->integer c))) char-set:full)))
-(test-equal #t (char-set=               ; only on Java 8
+(test-equal #t (char-set=               ; only on Java 9
                 char-set:letter
                 (char-set-filter
                  (lambda (c) (java.lang.Character:letter?
                               (char->integer c))) char-set:full)))
-(test-equal #t (char-set=               ; only on Java 8
+(test-equal #t (char-set=               ; only on Java 9
                 char-set:digit
                 (char-set-filter
                  (lambda (c) (java.lang.Character:digit?
@@ -665,7 +695,7 @@
               (= type java.lang.Character:FINAL_QUOTE_PUNCTUATION)
               (= type java.lang.Character:OTHER_PUNCTUATION))))
       char-set:full)))
-(test-equal                             ; only on Java 8
+(test-equal                             ; only on Java 9
  #t (char-set=
      char-set:symbol
      (char-set-filter
@@ -677,7 +707,7 @@
               (= type java.lang.Character:MODIFIER_SYMBOL)
               (= type java.lang.Character:OTHER_SYMBOL))))
       char-set:full)))
-(expect-fail-unless java-7 2)
+(expect-fail-unless java-9 2)
 (test-equal
  #t (char-set=
      char-set:whitespace
@@ -749,7 +779,7 @@
 
 (test-equal #f (char-set-any char-upper-case? char-set:lower-case))
 
-(expect-fail-unless java-8 1)
+(expect-fail-unless java-9 1)
 (test-equal #t (char-set-every char-upper-case? char-set:upper-case))
 ; char-set-adjoin, char-set-adjoin!
 (test-equal #t (char-set= abc
@@ -867,4 +897,16 @@
                                          intersection))))))
 (test-end)
 
+(import (srfi :13 strings))
+(test-equal 15 (string-contains "eek -- what a geek." "ee" 12 18))
+
+;;; Test SRFI-13 string-append/shared
+(let ((str "abc"))
+  (test-equal "" (string-append/shared))
+  (test-equal "" (string-append/shared ""))
+  (test-equal "abc" (string-append/shared str))
+  (set! str (string-append/shared str "123" "xy"))
+  (test-equal "abc123xy" (string-append/shared str))
+  (test-equal "abc123xy" str))
+
 (test-end)
